__author__ = 'DarkWeb'

import re

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup


# parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of description page
# return: 'row' that contains a variety of lists that each hold info on the description page
def bohemia_description_parser(soup):

    # Fields to be parsed

    vendor = "-1"                       # 0 *Vendor_Name
    success = "-1"                      # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"                # 2 Vendor_Rating
    name = "-1"                         # 3 *Product_Name
    describe = "-1"                     # 4 Product_Description
    CVE = "-1"                          # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                           # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"                     # 7 Product_Category
    views = "-1"                        # 8 Product_Number_Of_Views
    reviews = "-1"                      # 9 Product_Number_Of_Reviews
    rating_item = "-1"                  # 10 Product_Rating
    addDate = "-1"                      # 11 Product_AddedDate
    BTC = "-1"                          # 12 Product_BTC_SellingPrice
    USD = "-1"                          # 13 Product_USD_SellingPrice
    EURO = "-1"                         # 14 Product_EURO_SellingPrice
    sold = "-1"                         # 15 Product_QuantitySold
    left = "-1"                         # 16 Product_QuantityLeft
    shipFrom = "-1"                     # 17 Product_ShippedFrom
    shipTo = "-1"                       # 18 Product_ShippedTo

    # Finding Product Name
    name = soup.find('h1', {"style": "margin: 0; margin-bottom: 0.5em;"}).text
    name = name.replace('\n', ' ')
    name = name.replace(",", "")
    name = name.strip()

    # Finding Vendor
    vendor = soup.find('div', {"class": "user-photo"}).find_next_sibling('a').text
    vendor = vendor.strip()

    # Finding Vendor Rating
    rating_vendor = soup.find('span', {'class': "user-percent"}).text.strip()

    # Finding Users' Successful Transactions
    temp = ''
    success = soup.find('span', {'class': "smalltext shadow-text"}).text
    temp = success.split("|")
    success = str(temp[1])
    success = success.strip()

    # Finding Prices
    prices = soup.find('div', {'class': "col-md-3 sidebar-navigation user-details"}
                       ).find('div', {'class': "container text-left"})
    USD = prices.find('h1').text.strip()
    BTC = prices.find('h1').find_next_sibling('h3').text
    BTC = BTC.replace("BTC", "")
    BTC = BTC.strip()

    detail_row = soup.find('div', {'class': "detail-container text-left"}).find_all('strong')

    # Finding the Product Category (there isnt a thing for it on the page
    # category = li[1].find('span', {'class': "tag is-dark"}).text.strip()

    # Finding the Product Quantity Available
    left = soup.find('div', {'class': "container detail-container text-left"})
    left = left.find('div', {'class': "detail-row"}).text.replace('\n', '')
    left = left.split("Available Stock:")
    left = left[1].strip()

    # Finding Number Sold
    sold = detail_row[0].find_parent()
    sold = sold.text
    sold = sold.replace("Total Sold:", "")
    sold = sold.strip()

    # Finding Shipment Information (Origin) (There is no shipping information)
    '''if "Ships from:" in li[-2].text:
        shipFrom = li[-2].text
        shipFrom = shipFrom.replace("Ships from: ", "")
        # shipFrom = shipFrom.replace(",", "")
        shipFrom = shipFrom.strip()'''

    # Finding Shipment Information (Destination) (No shipping info
    '''shipTo = li[-1].find('div', {'title': "List of countries is scrollable"}).text
    shipTo = shipTo.replace("Ships to: ", "")
    shipTo = shipTo.strip()
    if "certain countries" in shipTo:
        countries = ""
        tags = li[-1].find_all('span', {'class': "tag"})
        for tag in tags:
            country = tag.text.strip()
            countries += country + ", "
        shipTo = countries.strip(", ")'''

    # Finding the Product description
    describe = soup.find('div', {'class': "container feedback-container"})
    describe = describe.find_next_sibling('div', {'class': "container"}).find('p').text
    describe = describe.replace("\n", " ")
    describe = describe.strip()

    # Finding the Number of Product Reviews
    review = detail_row[2].find_parent().text
    review = review.split("Based on")
    review = review[1].replace("ratings)", "").strip()

    # Searching for CVE and MS categories (cant find it)
    cve = soup.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
    if cve:
        CVE = " "
        for idx in cve:
            CVE += (idx)
            CVE += "  "
            CVE = CVE.replace(',', ' ')
            CVE = CVE.replace('\n', '')
    ms = soup.findAll(text=re.compile('MS\d{2}-\d{3}'))
    if ms:
        MS = " "
        for im in ms:
            MS += (im)
            MS += " "
            MS = MS.replace(',', ' ')
            MS = MS.replace('\n', '')

    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo)

    # Sending the results
    return row


# parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of listing page
# return: 'row' that contains a variety of lists that each hold info on the listing page
def bohemia_listing_parser(soup):

    # Fields to be parsed
    nm = 0                                    # *Total_Products (Should be Integer)
    mktName = "Bohemia"                       # 0 *Marketplace_Name
    vendor = []                               # 1 *Vendor y
    rating_vendor = []                        # 2 Vendor_Rating
    success = []                              # 3 Vendor_Successful_Transactions
    name = []                                 # 4 *Product_Name y
    CVE = []                                  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = []                                   # 6 Product_MS_Classification (Microsoft Security)
    category = []                             # 7 Product_Category y
    describe = []                             # 8 Product_Description
    views = []                                # 9 Product_Number_Of_Views
    reviews = []                              # 10 Product_Number_Of_Reviews
    rating_item = []                          # 11 Product_Rating
    addDate = []                              # 12 Product_AddDate
    BTC = []                                  # 13 Product_BTC_SellingPrice
    USD = []                                  # 14 Product_USD_SellingPrice y
    EURO = []                                 # 15 Product_EURO_SellingPrice
    sold = []                                 # 16 Product_QuantitySold
    qLeft =[]                                 # 17 Product_QuantityLeft
    shipFrom = []                             # 18 Product_ShippedFrom
    shipTo = []                               # 19 Product_ShippedTo
    href = []                                 # 20 Product_Links

    listing = soup.findAll('div', {"class": "product-link"})

    # Populating the Number of Products
    nm = len(listing)

    for a in listing:
        bae = a.findAll('a', href=True)

        # Adding the url to the list of urls
        link = bae[0].get('href')
        link = cleanLink(link)
        href.append(link)

        # Finding the Product
        product = bae[0].text
        product = product.replace('\n', ' ')
        product = product.replace(",", "")
        product = product.replace("...", "")
        product = product.strip()
        name.append(product)

        bae = a.find('div', {'class': "container"})

        # Finding Prices
        price = bae.find('div', {'class': "product-price"}).find('h2').text
        ud = price.replace("USD", " ")
        # u = ud.replace("$","")
        ud = ud.replace(",", "")
        ud = ud.strip()
        USD.append(ud)
        bc = bae.find('div', {'class': "product-price"}).find('span', {'class': "shadow-text smalltext boldtext"}).text
        bc = bc.replace("\n", "")
        bc = bc.split()
        bc = bc[0].replace("BTC", "").strip()
        BTC.append(bc)

        # Finding the Vendor
        vendor_name = bae.find('b').find('a').text
        vendor_name = vendor_name.strip()
        vendor.append(vendor_name)

        # Finding the Category
        cat = bae.find('span', {'class': "shadow-text smalltext"}).find('strong').text
        cat = cat.strip()
        category.append(cat)

        # Finding Number Sold and Quantity Left
        num = bae.find('div', {'class': "product-details-bottom"}).find('span').text
        num = num.replace("Sold", "")
        num = num.replace("times in total", "")
        num = num.strip()
        sold.append(num)

        quant = bae.find('div', {'class': "product-price"}).text
        quant = quant.replace("\n", "")
        quant = quant.split("Available")
        quant = quant[0].replace("Autoship", "").strip()
        qLeft.append(quant)

        # Finding Successful Transactions
        freq = bae.find('div', {'title': "Total Sales"}).find_parent().text.replace("\n", "")
        freq = freq.strip().split()
        freq = freq[-1].strip()
        success.append(freq)

        # find vendor rating
        rate = bae.find('b').find('strong').text.strip()
        rating_vendor.append(rate)

        # Searching for CVE and MS categories
        cve = a.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
        if not cve:
            cveValue = "-1"
        else:
            cee = " "
            for idx in cve:
                cee += (idx)
                cee += "  "
                cee = cee.replace(',', ' ')
                cee = cee.replace('\n', '')
            cveValue = cee
        CVE.append(cveValue)

        ms = a.findAll(text=re.compile('MS\d{2}-\d{3}'))
        if not ms:
            MSValue = "-1"
        else:
            me = " "
            for im in ms:
                me += (im)
                me += " "
                me = me.replace(',', ' ')
                me = me.replace('\n', '')
            MSValue = me
        MS.append(MSValue)

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href)


# called by the crawler to get description links on a listing page
# @param: beautifulsoup object that is using the correct html page (listing page)
# return: list of description links from a listing page
def bohemia_links_parser(soup):
    # Returning all links that should be visited by the Crawler

    href = []

    temp = soup.find('div', {"class": "col-md-9 sidebar-content-right listing-content"})
    temp = temp.find('div', {"class": "product-listing"})
    listing = temp.findAll('div', {"class": "product-heading"})

    for a in listing:
        bae = a.find('a', href=True)
        link = bae['href']
        href.append(link)

    return href