__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup


# parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of description page
# return: 'row' that contains a variety of lists that each hold info on the description page
def quest_description_parser(soup):
    # Fields to be parsed

    vendor = "-1"                       # 0 *Vendor_Name
    success = "-1"                      # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"                # 2 Vendor_Rating
    name = "-1"                         # 3 *Product_Name
    describe = "-1"                     # 4 Product_Description
    CVE = "-1"                          # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                           # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"                     # 7 Product_Category
    views = "-1"                        # 8 Product_Number_Of_Views
    reviews = "-1"                      # 9 Product_Number_Of_Reviews
    rating_item = "-1"                  # 10 Product_Rating
    addDate = "-1"                      # 11 Product_AddedDate
    BTC = "-1"                          # 12 Product_BTC_SellingPrice
    USD = "-1"                          # 13 Product_USD_SellingPrice
    EURO = "-1"                         # 14 Product_EURO_SellingPrice
    sold = "-1"                         # 15 Product_QuantitySold
    left = "-1"                         # 16 Product_QuantityLeft
    shipFrom = "-1"                     # 17 Product_ShippedFrom
    shipTo = "-1"                       # 18 Product_ShippedTo
    image = "-1"                        # 19 Product_Image
    vendor_image = "-1"                 # 20 Vendor_Image

    # Finding Product Name
    name = soup.find('div', class_='card-header bg-dark text-white rounded-0 text-center').text.strip()

    # USD Price
    USD = soup.find('small', text='Product Price:').find_next('small').text.strip().replace('$', '')

    # Product Description
    describe = soup.find('textarea').text.strip()

    # Finding Product Image
    image = soup.find('img', {'class': 'img-fluid'})
    image = image.get('src').split('base64,')[-1]

    # Finding Vendor Image
    vendor_image = soup.select_one('.card-body.bg-mgray.css-selector.shadow img')
    vendor_image = vendor_image.get('src').split('base64,')[-1]


    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo, image, vendor_image)

    # Sending the results
    return row


# parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
# stores info it needs in different lists, these lists are returned after being organized
# @param: soup object looking at html page of listing page
# return: 'row' that contains a variety of lists that each hold info on the listing page
def quest_listing_parser(soup):

    # Fields to be parsed
    nm = 0  # *Total_Products (Should be Integer)
    mktName = "quest"                     # 0 *Marketplace_Name
    vendor = []                               # 1 *Vendor y
    rating_vendor = []                        # 2 Vendor_Rating
    success = []                              # 3 Vendor_Successful_Transactions
    name = []                                 # 4 *Product_Name y
    CVE = []                                  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures) dont worry about this
    MS = []                                   # 6 Product_MS_Classification (Microsoft Security) dont worry about this
    category = []                             # 7 Product_Category y
    describe = []                             # 8 Product_Description
    views = []                                # 9 Product_Number_Of_Views
    reviews = []                              # 10 Product_Number_Of_Reviews
    rating_item = []                          # 11 Product_Rating
    addDate = []                              # 12 Product_AddDate
    BTC = []                                  # 13 Product_BTC_SellingPrice
    USD = []                                  # 14 Product_USD_SellingPrice y
    EURO = []                                 # 15 Product_EURO_SellingPrice
    sold = []                                 # 16 Product_QuantitySold
    qLeft = []                                # 17 Product_QuantityLeft
    shipFrom = []                             # 18 Product_ShippedFrom
    shipTo = []                               # 19 Product_ShippedTo
    image = []                                # 20 Product_Image
    image_vendor = []                         # 21 Vendor_Image
    href = []                                 # 22 Product_Links

    # Extract all product listings
    listing = soup.findAll('div', class_='col-md-2 my-md-0 col-12')
    # Populating the Number of Products
    nm = len(listing)

    for a in listing:

        # Extracting Product URL & Name
        product_link_tags = a.find_all('a', class_='badge-info')
        if product_link_tags:
            # Using the first tag as default
            product_link_tag = product_link_tags[0]
            href.append(product_link_tag['href'])
            name.append(product_link_tag.text.strip())

        # Extracting Product Image
        img_tag = a.find('img')
        if img_tag:
            image_data = img_tag['src'].split('base64,')[-1]
            image.append(image_data)

        # Extracting Vendor Name
        vendor_tag = a.find('a', class_='badge-dark')
        if vendor_tag:
            vendor.append(vendor_tag.text.replace('👤', '').strip())

        # Extracting Product Price in USD
        price_tag = a.find('a', class_='text')
        if price_tag:
            USD.append(price_tag.text.replace("$", "").strip())

        category_tag = soup.find('span', class_= 'btn btn-sm btn-outline-mgray active border-info')
        if category_tag:
            category.append(category_tag.text.strip())


    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href, image, image_vendor)


# called by the crawler to get description links on a listing page
# @param: beautifulsoup object that is using the correct html page (listing page)
# return: list of description links from a listing page
def quest_links_parser(soup):
    # Returning all product links

    href = []

    # Locate all divs with class 'row'
    row_divs = soup.findAll('div', class_='row')

    for row_div in row_divs:
        # Locate all product divs within the current 'row' div
        product_divs = row_div.findAll('div', class_='col-md-2 my-md-0 col-12')

        for product_div in product_divs:
            # Locate the anchor tag containing the product link within each product div
            product_link_tag = product_div.find('a', class_='badge-info')

            if product_link_tag and product_link_tag.has_attr('href'):
                href.append(product_link_tag['href'])

    return href